"use strict";
var lodash_1 = require('lodash');
var ObservableQuery = (function () {
    function ObservableQuery(queryManager, queryId, options) {
        this.isLoading = true;
        this.queryManager = queryManager;
        this.queryId = queryId;
        this.options = options;
        this.observers = [];
    }
    ObservableQuery.prototype.subscribe = function (observer) {
        var _this = this;
        this.observers.push(observer);
        if (this.observers.length == 1) {
            this.queryManager.registerObservedQuery(this);
        }
        this.refetch();
        return {
            unsubscribe: function () {
                lodash_1.pull(_this.observers, observer);
                if (_this.observers.length < 1) {
                    _this.queryManager.deregisterObservedQuery(_this);
                }
            },
        };
    };
    ObservableQuery.prototype.result = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var subscription = _this.subscribe({
                next: function (result) {
                    resolve(result);
                    setTimeout(function () {
                        subscription.unsubscribe();
                    }, 0);
                },
                error: function (error) {
                    reject(error);
                },
            });
        });
    };
    ObservableQuery.prototype.refetch = function () {
        this.queryManager.fetchQuery(this);
    };
    ObservableQuery.prototype.didReceiveResult = function (result) {
        this.observers.forEach(function (observer) {
            observer.next(result);
        });
    };
    ObservableQuery.prototype.didReceiveError = function (error) {
        this.observers.forEach(function (observer) {
            observer.error(error);
        });
    };
    return ObservableQuery;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = ObservableQuery;
//# sourceMappingURL=ObservableQuery.js.map